@@ -61,68 +61,71 @@ setMethod(
 	signature = 'mirtClass',
 	definition = function(object, full.scores = FALSE, method = "EAP", ...)
 	{    
-		rotF <- object@F
-		cs <- sqrt(1 - object@h2)
-		a <- as.matrix(rotF / cs)
-		d <- qnorm(object@facility) / cs  
-		g <- object@guess  
+		K <- object@K		
+		a <- object@pars$lambdas		
+		d <- object@pars$zetas		
+		g <- object@guess				
+		itemloc <- object@itemloc
+		J <- nrow(a)
 		nfact <- ncol(a)
 		theta <- as.matrix(seq(-4,4,length.out = 15))
 		Theta <- thetaComb(theta,nfact)
-		fulldata <- object@fulldata  
-		tabdata <- object@tabdata[ ,1:ncol(fulldata)]
-		colnames(tabdata) <- colnames(fulldata) 
-		SEscores <- scores <- matrix(0,ncol=ncol(Theta),nrow=nrow(tabdata))
-		SE <- thetas <- rep(0,nfact)
-		W <- dmvnorm(Theta,rep(0,nfact),diag(nfact))    
+		fulldata <- object@data 
+		tabdata <- object@tabdatalong
+		tabdata <- tabdata[,-ncol(tabdata)]
+		SEscores <- scores <- matrix(0, nrow(tabdata), nfact)			
+		W <- mvtnorm::dmvnorm(Theta,rep(0,nfact),diag(nfact)) 
 		W <- W/sum(W)
-		for (i in 1:nrow(scores)){
-			L <- 0  
-			for (j in 1:nrow(a)){
-				if(tabdata[i,j] == 1) L <- log(P.mirt(a[j, ],d[j],Theta,g[j])) + L
-				else L <- log(1 - P.mirt(a[j, ],d[j],Theta,g[j])) + L	
-			}	
-			for (k in 1:ncol(Theta))
-				thetas[k] <- sum(Theta[ ,k] * exp(L) * W / sum(exp(L) * W))
-			for (k in 1:ncol(Theta))	
-				SE[k] <- sqrt(sum((Theta[,k] - thetas[k])^2 * exp(L) * W / sum(exp(L) * W)))
+		itemtrace <- matrix(0, ncol=ncol(tabdata), nrow=nrow(Theta))
+		for (i in 1:J){
+			if(length(d[[i]]) == 1){
+				itemtrace[ ,itemloc[i]] <- P.mirt(a[i, ], d[[i]], Theta, g[i]) 
+				itemtrace[ ,itemloc[i] + 1] <- 1.0 - itemtrace[ ,itemloc[i]]
+			} else {
+				itemtrace[ ,itemloc[i]:(itemloc[i+1] - 1)] <- 
+					P.poly(a[i, ], d[[i]], Theta, TRUE)	
+			}
+		}			
+		for (i in 1:nrow(tabdata)){				
+			L <- rowSums(log(itemtrace)[ ,as.logical(tabdata[i,])])			
+			thetas <- colSums(Theta * exp(L) * W / sum(exp(L) * W))
+			SE <- sqrt(colSums((Theta - thetas)^2 * exp(L) * W / sum(exp(L) * W)))	
 			scores[i, ] <- thetas
 			SEscores[i, ] <- SE
 		}		
-		if(method == "MAP"){
+		if(method == "MAP"){ 
 			for (i in 1:nrow(scores)){       
 				Theta <- scores[i, ]	  
-				thetas <- nlm(MAP.mirt,Theta,a=a,d=d,guess=g,patdata=tabdata[i, ])$estimate 
+				thetas <- nlm(MAP.mirt,Theta,a=a,d=d,guess=g,patdata=tabdata[i, ],itemloc)$estimate 
 				scores[i, ] <- thetas
 			}  
 		}
 		if(method == "ML"){
-            scores[rowSums(tabdata) == 0 | rowSums(tabdata) == ncol(fulldata), ] <- NA
-		for (i in 1:nrow(scores)){
-            if(any((scores[i, ]) == -Inf | scores[i, ] == Inf)) next 
-		        Theta <- scores[i, ]	  
-		        thetas <- nlm(MAP.mirt,Theta,a=a,d=d,guess=g,patdata=tabdata[i, ],ML=TRUE)$estimate 
-		        scores[i, ] <- thetas
-		  }  
+			tmp <- tabdata[,itemloc[-length(itemloc)]]			 
+			tmp2 <- tabdata[,itemloc[-1] - 1]			 
+            scores[rowSums(tmp) == J, ] <- Inf
+			scores[rowSums(tmp2) == J,] <- -Inf
+			for (i in 1:nrow(scores)){
+				if(any((scores[i, ]) == -Inf | scores[i, ] == Inf)) next 
+				Theta <- scores[i, ]	  
+				thetas <- nlm(MAP.mirt,Theta,a=a,d=d,guess=g,patdata=tabdata[i, ],itemloc,ML=TRUE)$estimate 
+				scores[i, ] <- thetas
+			}  
 		}
 		colnames(scores) <- colnames(object@F)
 		if (full.scores){      
-			scoremat <- matrix(0,nrow=nrow(fulldata),ncol=ncol(Theta)) 
+			scoremat <- matrix(0,nrow=nrow(fulldata),ncol=ncol(Theta))
+			tabdata2 <- object@tabdata[,-(ncol(fulldata)+1)]	
 			for (j in 1:nrow(tabdata)){          
-				TFvec <- colSums(ifelse(t(fulldata) == tabdata[j, ],1,0)) == ncol(fulldata)        
+				TFvec <- colSums(ifelse(t(fulldata) == tabdata2[j, ],1,0)) == ncol(fulldata)        
 				scoremat[TFvec, ] <- scores[j, ]
 			} 
 			colnames(scoremat) <- colnames(object@F)	
 			return(cbind(fulldata,scoremat))
-		} else {
-			r <- matrix(object@tabdata[ ,ncol(tabdata)+1])
-			colnames(r) <- 'Freq'			
-			colnames(SEscores) <- paste("SE_",colnames(scores),sep='')
+		} else {						
 			cat("\nMethod: ", method,"\n\n")
-			if(method == "EAP")	
-				return(cbind(tabdata,r,scores,SEscores))
-			else
-				return(cbind(tabdata,r,scores))	
+			return(cbind(object@tabdata,scores))
+				
 		}   
 	}  
 )
@@ -131,49 +134,60 @@ setMethod(
 setMethod(
 	f = "fscores",
 	signature = 'bfactorClass',
-	definition = function(object, full.scores = FALSE, method = "EAP", ...){  
-		g <- object@guess
-		d <- object@pars[ ,ncol(object@pars)]
-		a <- as.matrix(object@pars[ ,1:(ncol(object@pars) - 1)])
+	definition = function(object, full.scores = FALSE, method = "EAP", ...)
+	{  
+		K <- object@K		
+		a <- object@pars$lambdas		
+		d <- object@pars$zetas		
+		g <- object@guess				
+		itemloc <- object@itemloc
+		J <- nrow(a)
 		nfact <- 2
 		theta <- as.matrix(seq(-4,4,length.out = 15))
 		Theta <- thetaComb(theta,nfact)
-		fulldata <- object@fulldata  
-		tabdata <- object@tabdata[ ,1:ncol(fulldata)]
-		colnames(tabdata) <- colnames(fulldata) <- object@itemnames
+		fulldata <- object@data 
+		tabdata <- object@tabdatalong
+		tabdata <- tabdata[,-ncol(tabdata)]
 		SEscores <- scores <- rep(0,nrow(tabdata))
 		SE <- thetas <- 0
 		logicalfact <- object@logicalfact
-		W <- dmvnorm(Theta,rep(0,nfact),diag(nfact)) 
-		W <- W/sum(W)	
-		for (i in 1:nrow(tabdata)){
-			L <- 0  
-			for (j in 1:nrow(a)){
-				if(tabdata[i,j] == 1) 
-					L <- log(P.bfactor(a[j, ],d[j],Theta,g[j],logicalfact[j, ])) + L
-				else 
-					L <- log(1 - P.bfactor(a[j, ],d[j],Theta,g[j],logicalfact[j, ])) + L	
-			}	
+		W <- mvtnorm::dmvnorm(Theta,rep(0,nfact),diag(nfact)) 
+		W <- W/sum(W)
+		itemtrace <- matrix(0, ncol=ncol(tabdata), nrow=nrow(Theta))
+		for (i in 1:J){
+			if(length(d[[i]]) == 1){
+				itemtrace[ ,itemloc[i]] <- P.bfactor(a[i, ], d[[i]], Theta, g[i], logicalfact[i, ]) 
+				itemtrace[ ,itemloc[i] + 1] <- 1.0 - itemtrace[ ,itemloc[i]]
+			} else {
+				itemtrace[ ,itemloc[i]:(itemloc[i+1] - 1)] <- 
+					P.bfactor(a[i, ], d[[i]], Theta, 0, logicalfact[i, ])	
+			}
+		}		
+		for (i in 1:nrow(tabdata)){							
+			L <- rowSums(log(itemtrace)[ ,as.logical(tabdata[i,])])				
 			thetas <- sum(Theta[ ,1] * exp(L) * W / sum(exp(L) * W))
 			SE <- sqrt(sum((Theta[ ,1] - thetas)^2 * exp(L) * W / sum(exp(L) * W)))	
 			scores[i] <- thetas
 			SEscores[i] <- SE
 		}
-		if(method == "MAP"){
+		if(method == "MAP"){ 
 			for (i in 1:length(scores)) {       
 				Theta <- scores[i]	  
 				thetas <- nlm(MAP.bfactor,Theta,a=a,d=d,guess=g,
-					patdata=tabdata[i, ],logicalfact=logicalfact)$estimate 
+					patdata=tabdata[i, ],logicalfact=logicalfact,itemloc)$estimate 
 				scores[i] <- thetas
 			}  
 		}
-		if(method == "ML"){
-		    scores[rowSums(tabdata) == 0 | rowSums(tabdata) == ncol(fulldata)] <- NA
+		if(method == "ML"){			
+		    tmp <- tabdata[,itemloc[-length(itemloc)]]			 
+			tmp2 <- tabdata[,itemloc[-1] - 1]
+			scores[rowSums(tmp) == J, ] <- Inf
+			scores[rowSums(tmp2) == J,] <- -Inf
 		    for (i in 1:length(scores)) { 
 		        if(any(scores[i] == -Inf | scores[i] == Inf)) next
 		        Theta <- scores[i]	  
 		        thetas <- nlm(MAP.bfactor,Theta,a=a,d=d,guess=g,
-		                      patdata=tabdata[i, ],logicalfact=logicalfact,ML=TRUE)$estimate 
+		                      patdata=tabdata[i, ],logicalfact=logicalfact,itemloc,ML=TRUE)$estimate 
 		        scores[i] <- thetas
 		    }
 		}
@@ -185,21 +199,17 @@ setMethod(
 			colnames(scores) <- 'g'
 		}	
 		if(full.scores){
-			scoremat <- matrix(0,nrow=nrow(fulldata),ncol=1) 
+			scoremat <- matrix(0,nrow=nrow(fulldata),ncol=ncol(Theta))
+			tabdata2 <- object@tabdata[,-(ncol(fulldata)+1)]	
 			for (j in 1:nrow(tabdata)){          
-				TFvec <- colSums(ifelse(t(fulldata) == tabdata[j, ],1,0)) == ncol(fulldata)        
-				scoremat[TFvec, ] <- scores[j,1]
+				TFvec <- colSums(ifelse(t(fulldata) == tabdata2[j, ],1,0)) == ncol(fulldata)        
+				scoremat[TFvec, ] <- scores[j, ]
 			} 
-			colnames(scoremat) <- colnames(object@F)
+			colnames(scoremat) <- colnames(object@F)	
 			return(cbind(fulldata,scoremat))
-		} else {  	
-			r <- matrix(object@tabdata[,ncol(tabdata)+1])
-			colnames(r) <- 'Freq'
-			cat("\nMethod: ", method,"\n\n")
-			if(method == "EAP")	
-				return(cbind(tabdata,r,scores))
-			else
-				return(cbind(tabdata,r,scores))	
+		} else {  				
+			cat("\nMethod: ", method,"\n\n")			
+			return(cbind(object@tabdata,scores))			
 		}   
 	}  
 )
@@ -214,8 +224,8 @@ setMethod(
 		theta0 <- object@Theta
 		K <- object@K
 		nfact <- ncol(theta0)
-		lambdas <- matrix(object@pars[,1:nfact],ncol=nfact)
-		zetas <- na.omit(as.numeric(t(object@pars[,(nfact+1):ncol(object@pars)])))
+		lambdas <- object@pars$lambdas
+		zetas <- object@pars$zetas
 		guess <- object@guess
 		guess[is.na(guess)] <- 0
 		data <- cbind(object@data,object@fulldata)
@@ -234,8 +244,7 @@ setMethod(
 		}
 		ind <- 1
 		for(i in 1:ndraws){			
-			theta0 <- draw.thetas(theta0,lambdas,zetas,guess,tabdata,K,itemloc,cand.t.var)
-			theta0[CONSTRAIN, ] <- 0
+			theta0 <- draw.thetas(theta0,lambdas,zetas,guess,tabdata,K,itemloc,cand.t.var)			
 			if(i %% thin == 0){
 				for(j in 1:nfact)
 					Theta[[j]][,ind] <- theta0[,j]									
@@ -248,9 +257,7 @@ setMethod(
 		for(i in 1:nfact){
 			expscores[,i] <- rowMeans(Theta[[i]])
 			sdscores[,i] <- apply(Theta[[i]],1,sd)
-		}
-        expscores[CONSTRAIN] <- NA
-		sdscores[CONSTRAIN] <- NA
+		}        
 				
 		ret <- cbind(unique(data)[,1:length(K)],expscores,sdscores)
 		colnames(ret) <- Names
@@ -294,8 +301,8 @@ setMethod(
 		guess <- object@guess
 		guess[is.na(guess)] <- 0
 		data <- cbind(object@data,object@fulldata)		
-		Names <- c(colnames(object@data[,1:length(K)]),factorNames,
-			paste("SE_",factorNames,sep=''))
+		Names <- c(colnames(object@data[,1:length(K)]),paste('F_',1:nfact,sep=''),
+			paste("SE_",1:nfact,sep=''))
 		tabdata <- unique(data)[,-c(1:length(K))]			
 		itemloc <- object@itemloc
 		Theta <- list()
